:root {
  --md-primary-fg-color: rgb(245, 192, 59);
  --md-primary-fg-color--light: #ecb7b7;
  --md-primary-fg-color--dark: #90030c;
  --md-accent-fg-color: rgb(245, 192, 59);
  --md-accent-fg-color--light: #ecb7b7;
  --md-accent-fg-color--dark: #90030c;
}

.md-header {
  background-color: var(--md-primary-fg-color);
  color: black;
}
.md-search__input + .md-search__icon {
  color: black;
}
.md-search__input::placeholder {
  color: black;
}
.md-nav__link[for]:focus,
.md-nav__link[for]:hover,
.md-nav__link[href]:focus,
.md-nav__link[href]:hover {
  text-decoration: underline;
}

.md-typeset a:focus,
.md-typeset a:hover {
  text-decoration: underline;
}

header {
  border-bottom: 4px solid black;
}

.md-copyright {
  width: 100%;
}

.md-copyright .made-with-love {
  float: right;
  line-height: 24px;
}

.md-copyright .made-with-love a img {
  height: 24px;
  vertical-align: top;
  opacity: 0.7;
  transition: all 0.3s ease-in-out;
}

.md-copyright .made-with-love a:hover img {
  opacity: 1;
}

tr.benchmarks-best-result {
  background-color: var(--md-typeset-ins-color);
}

tr.benchmarks-worst-result {
  background-color: var(--md-typeset-del-color);
}

@keyframes heart {
  0%,
  40%,
  80%,
  100% {
    transform: scale(1);
  }
  20%,
  60% {
    transform: scale(1.15);
  }
}
.heart {
  animation: heart 1000ms infinite;
  color: #90030c;
}

.main-logo {
    padding: 25px;
}
.main-logo path {
  fill: var(--md-primary-fg-color);
}
